@@ -40,8 +40,6 @@
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 
-#include <libedataserver/e-msgport.h>
-
 #include <e-gw-connection.h>
 #include <e-gw-item.h>
 
@@ -453,6 +451,20 @@ groupwise_folder_search_by_expression (CamelFolder *folder, const char *expressi
 	return matches;
 }
 
+static guint32
+groupwise_folder_count_by_expression (CamelFolder *folder, const char *expression, CamelException *ex)
+{
+	CamelGroupwiseFolder *gw_folder = CAMEL_GROUPWISE_FOLDER(folder);
+	guint32 matches;
+
+	CAMEL_GROUPWISE_FOLDER_LOCK(gw_folder, search_lock);
+	camel_folder_search_set_folder (gw_folder->search, folder);
+	matches = camel_folder_search_count (gw_folder->search, expression, ex);
+	CAMEL_GROUPWISE_FOLDER_UNLOCK(gw_folder, search_lock);
+
+	return matches;
+}
+
 static GPtrArray *
 groupwise_folder_search_by_uids(CamelFolder *folder, const char *expression, GPtrArray *uids, CamelException *ex)
 {
@@ -674,6 +686,7 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 
 				gw_info->info.flags &= ~CAMEL_MESSAGE_FOLDER_FLAGGED;
 				gw_info->info.dirty = 1;
+				camel_folder_summary_touch (folder->summary);
 				gw_info->server_flags = gw_info->info.flags;
 				uid = camel_message_info_uid (info);
 				if (diff.bits & CAMEL_MESSAGE_DELETED) {
@@ -1761,7 +1774,7 @@ groupwise_folder_item_to_msg( CamelFolder *folder,
 
 
 	msg = camel_mime_message_new ();
-	if (has_mime_822) {
+	if (has_mime_822 && body) {
 		temp_stream = camel_stream_mem_new_with_buffer (body, body_len);
 		if (camel_data_wrapper_construct_from_stream ((CamelDataWrapper *) msg, temp_stream) == -1) {
 			camel_object_unref (msg);
@@ -2397,6 +2410,7 @@ camel_groupwise_folder_class_init (CamelGroupwiseFolderClass *camel_groupwise_fo
 	camel_folder_class->get_message = groupwise_folder_get_message;
 	camel_folder_class->rename = groupwise_folder_rename;
 	camel_folder_class->search_by_expression = groupwise_folder_search_by_expression;
+	camel_folder_class->count_by_expression = groupwise_folder_count_by_expression;
 	camel_folder_class->search_by_uids = groupwise_folder_search_by_uids; 
 	camel_folder_class->search_free = groupwise_folder_search_free;
 	camel_folder_class->append_message = groupwise_append_message;
